<?php

namespace Crm\Customer\Listeners;

use Illuminate\Support\Facades\Notification;
use Illuminate\Support\Str;
use Crm\Customer\Events\CustomerWorkflow as CustomerWorkflowEvent;
use Crm\Customer\Notifications\CustomerWorkflow as CustomerWorkflowNotification;
use Litepie\Actions\Concerns\AsAction;

class CustomerWorkflow
{
    use AsAction;

    private $allowedTransitions = [
        'before' => [],
        'after' => ['publish', 'submit', 'approve'],
    ];

    public function handle(CustomerWorkflowEvent $event)
    {
        $function = Str::camel($event->transition);
        return $this->$function($event);
    }

    /**
     * Sends a notification to the client when the $customer is submitted.
     *
     * @param CustomerWorkflowEvent $event The event object.
     * @return void
     */
    public function submit(CustomerWorkflowEvent $event)
    {
        $client = $event->customer->client;
        Notification::send($client, new CustomerWorkflowNotification($event));
    }

    /**
     * Sends a notification to the client when the $customer is published.
     *
     * @param CustomerWorkflowEvent $event The event object.
     * @return void
     */
    public function publish(CustomerWorkflowEvent $event)
    {
        $client = $event->customer->client;
        Notification::send($client, new CustomerWorkflowNotification($event));
    }

    /**
     * Sends a notification to the client when the $customer is approved.
     *
     * @param CustomerWorkflowEvent $event The event object.
     * @return void
     */
    public function approve(CustomerWorkflowEvent $event)
    {
        $client = $event->customer->client;
        Notification::send($client, new CustomerWorkflowNotification($event));
    }

    /**
     * Handles the $customer workflow event as a listener.
     *
     * @param CustomerWorkflowEvent $event The event object.
     * @return void
     */
    public function asListener(CustomerWorkflowEvent $event)
    {
        if (($event->when == 'before' &&
            in_array($event->transition, $this->allowedTransitions['before']) ||
            $event->when == 'after' &&
            in_array($event->transition, $this->allowedTransitions['after']))
        ) {
            return $this->handle($event);
        }
    }
}
