<?php

namespace Crm\Customer\Listeners;

use Illuminate\Support\Facades\Notification;
use Illuminate\Support\Str;
use Crm\Customer\Events\WebsiteWorkflow as WebsiteWorkflowEvent;
use Crm\Customer\Notifications\WebsiteWorkflow as WebsiteWorkflowNotification;
use Litepie\Actions\Concerns\AsAction;

class WebsiteWorkflow
{
    use AsAction;

    private $allowedTransitions = [
        'before' => [],
        'after' => ['publish', 'submit', 'approve'],
    ];

    public function handle(WebsiteWorkflowEvent $event)
    {
        $function = Str::camel($event->transition);
        return $this->$function($event);
    }

    /**
     * Sends a notification to the client when the $website is submitted.
     *
     * @param WebsiteWorkflowEvent $event The event object.
     * @return void
     */
    public function submit(WebsiteWorkflowEvent $event)
    {
        $client = $event->website->client;
        Notification::send($client, new WebsiteWorkflowNotification($event));
    }

    /**
     * Sends a notification to the client when the $website is published.
     *
     * @param WebsiteWorkflowEvent $event The event object.
     * @return void
     */
    public function publish(WebsiteWorkflowEvent $event)
    {
        $client = $event->website->client;
        Notification::send($client, new WebsiteWorkflowNotification($event));
    }

    /**
     * Sends a notification to the client when the $website is approved.
     *
     * @param WebsiteWorkflowEvent $event The event object.
     * @return void
     */
    public function approve(WebsiteWorkflowEvent $event)
    {
        $client = $event->website->client;
        Notification::send($client, new WebsiteWorkflowNotification($event));
    }

    /**
     * Handles the $website workflow event as a listener.
     *
     * @param WebsiteWorkflowEvent $event The event object.
     * @return void
     */
    public function asListener(WebsiteWorkflowEvent $event)
    {
        if (($event->when == 'before' &&
            in_array($event->transition, $this->allowedTransitions['before']) ||
            $event->when == 'after' &&
            in_array($event->transition, $this->allowedTransitions['after']))
        ) {
            return $this->handle($event);
        }
    }
}
